 title  "TEST1 - Turn On/Show LED Segments."
;
;  This Program simply Displays each of the Segments in the 3-Digit
;   LED Thermometer Routine.  Each Segment is output in order, as
;   defined below:
;
;            --6--
;           |     |
;           5     1
;           |     |
;            --7--
;           |     |
;           4     2
;           |     |
;            --3--
;
;  Each LED Segment is connected to a PIC pin via a 220 Ohm Resistor
;   Except for the Segment connected to RA0, in which the 220 Ohm Resistor
;   is attached to Vdd and the PIC Pin pulls it low to turn OFF the LED.
;
;  The operation of each different 7 Segment Display is controlled by a
;   2N70000 N-Channel MOSFET attached to the common cathod to the display.
;   The gate to the controlling MOSFET is connected directly to an output
;   PIN on the PIC.
;
;  Hardware Notes:
;   Reset is tied directly to Vcc and PWRT is Enabled.
;   RA2 is Connected to Segment 6
;   RA3 is Connected to Segment 5
;   RA4 is Connected to Segment 1
;   RB0 is Connected to Segment 7
;   RB1 is Connected to Segment 4
;   RB2 is Connected to Segment 3
;   RB3 is Connected to Segment 2
;   RB4 is Connected to the Left Most Digit
;   RB5 is Connected to the Middle Digit
;   RB6 is Connected to the Right Most Digit
;
;  Myke Predko
;  96.07.20
;
  LIST P=16C84, R=DEC
  errorlevel 0,-305
  INCLUDE "d:\writing\pic_mast\code\p16cxx.inc"

;  Registers
Active_Seg EQU  0x00C           ;  Segment of the Display Being Shown
Active_Dig EQU  0x00D           ;  Segment Currently Being Displayed
Count   EQU     0x00E
Counthi EQU     0x00F

 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF
                                ;  Note that the WatchDog Timer is OFF

  PAGE
;  Code for TEST1

  org    0

  goto   MainLine               ;  Skip Over Output Tables

SEGMENTS_A                      ;  PORTA Outputs for the Segments
  addwf  PCL
  retlw  0x010                  ;  Segment "1"
  retlw  0                      ;  Segment "2"
  retlw  0                      ;  Segment "3"
  retlw  0                      ;  Segment "4"
  retlw  8                      ;  Segment "5"
  retlw  4                      ;  Segment "6"
  retlw  0                      ;  Segment "7"

SEGMENTS_B                      ;  PORTB Outputs for the Segments
  addwf  PCL
  retlw  0                      ;  Segment "1"
  retlw  8                      ;  Segment "2"
  retlw  4                      ;  Segment "3"
  retlw  2                      ;  Segment "4"
  retlw  0                      ;  Segment "5"
  retlw  0                      ;  Segment "6"
  retlw  1                      ;  Segment "7"

Digits                          ;  Output Digits
  addwf  PCL
  retlw  0x040                  ;  First Digit
  retlw  0x020                  ;  Second Digit
  retlw  0x010                  ;  Third Digit

MainLine

  clrf   Active_Seg             ;  Start at the Beginning for Displaying
  clrf   Active_Dig

  clrf   PORTA                  ;  Make Sure the Ports can't Output Data
  clrf   PORTB

  bsf    STATUS, RP0            ;  Enable the Value Change LED
  movlw  0x03                   ;  Enable Output on PORTA.4:2
  movwf  TRISA & 0x07F
  movlw  0x080                  ;  Enable All Outputs except PORTB.7
  movwf  TRISB & 0x07F
  bcf    STATUS, RP0

Loop                            ;  Loop Around Here to Output the Sawtooth

  movf   Active_Seg, w          ;  Turn Off Digit O/P and Turn On Display
  call   SEGMENTS_B
  movwf  PORTB

  movf   Active_Seg, w          ;  Now, Make Sure PORTA is Set Correctly
  call   SEGMENTS_A
  movwf  PORTA

  movf   Active_Dig, w          ;  Now, Everything Setup, Turn on the Digit
  call   Digits
  iorwf  PORTB

  ifndef Debug                  ;  During Debug, Don't Wait 1/2 Second
   call  Dlay
  endif

  incf   Active_Seg             ;  Point to the Next Segment
  movlw  7                      ;  Have we shown them all on the Digit?
  subwf  Active_Seg, w
  btfss  STATUS, C              ;  If Result is Positive, Then We have
   goto  Loop

  clrf   Active_Seg             ;  Point to the Next Segment

  incf   Active_Dig             ;  Now, Do the Same for the Digits
  movlw  3                      ;  Are we at the Last One?
  subwf  Active_Dig, w
  btfsc  STATUS, C              ;  If Result is Negative, Don't Clear
   clrf  Active_Dig

  goto   Loop


Dlay

  movlw  194                    ;  Now, Display for a Half a Second
  movwf  Count
  movlw  163
  movwf  Counthi
Dlay_Loop
  decfsz Count
   goto  Dlay_Loop
  decfsz Counthi
   goto  Dlay_Loop

  return

  end
