 title  "TEST6 - Read a POT and Show the Temp."
;
;  This Program Reads a POT (Using the Method used by the Basic Stamp
;   II "RCTIME" Instruction) and then converts the Temperature to Celcius
;   using a Table (with Values Provided by the component Specification).
;
;  This Program assumes ideal components (ie 0% tolerance on the Resistor
;   and Capacitor and no leakage back through the PIC).  The actual values
;   displayed *may* be in Error.
;
;            --6--
;           |     |
;           5     1
;           |     |
;            --7--
;           |     |
;           4     2
;           |     |
;            --3--
;
;  Each LED Segment is connected to a PIC pin via a 220 Ohm Resistor
;   Except for the Segment connected to RA0, in which the 220 Ohm Resistor
;   is attached to Vdd and the PIC Pin pulls it low to turn OFF the LED.
;
;  The operation of each different 7 Segment Display is controlled by a
;   2N70000 N-Channel MOSFET attached to the common cathod to the display.
;   The gate to the controlling MOSFET is connected directly to an output
;   PIN on the PIC.
;
;  Hardware Notes:
;   Reset is tied directly to Vcc and PWRT is Enabled.
;   RA2 is Connected to Segment 6
;   RA3 is Connected to Segment 5
;   RA4 is Connected to Segment 1
;   RB0 is Connected to Segment 7
;   RB1 is Connected to Segment 4
;   RB2 is Connected to Segment 3
;   RB3 is Connected to Segment 2
;   RB4 is Connected to the Left Most Digit
;   RB5 is Connected to the Middle Digit
;   RB6 is Connected to the Right Most Digit
;   RB7 is Connected to the RC Network
;
;  The Various Numbers use the Segments Below:
;   "0" - Segments 1, 2, 3, 4, 5, 6  Ports RA4, RB3, RB2, RB1, RA3, RA2
;   "1" - Segments 1, 2              Ports RA4, RB3
;   "2" - Segments 6, 1, 7, 4, 3     Ports RA2, RA4, RB0, RB1, RB2
;   "3" - Segments 6, 1, 2, 3, 7     Ports RA2, RA4, RB3, RB2, RB0
;   "4" - Segmemts 7, 1, 2, 5        Ports RB0, RA4, RB3, RA3
;   "5" - Segments 6, 5, 7, 2, 3     Ports RA2, RA3, RB0, RB3, RB2
;   "6" - Segments 6, 5, 4, 3, 2, 7  Ports RA2, RA3, RB1, RB2, RB3, RB0
;   "7" - Segments 6, 1, 2           Ports RA2, RA4, RB3
;   "8" - ALL Segments               ALL Port Bits
;   "9" - Segments 6, 1, 2, 7, 5     Ports RA2, RA4, RB3, RB0, RA3
;   "C" - Segments 6, 5, 4, 3        Ports RA2, RA3, RB1, RB2
;   "A" - Segments 6, 1, 2, 4, 5, 7  Ports RA2, RA4, RB3, RB1, RA3, RB0
;   "L" - Segments 5, 4, 3           Ports RA3, RB1, RB2
;   "u" - Segments 2, 3, 4           Ports RB3, RB2, RB1
;   "-" - Segments 7                 Ports RB0
;   "^" - Segments 1, 5, 6           Ports RA4, RA3, RA2
;
;  Myke Predko
;  96.07.21
;
  LIST P=16C84, R=DEC
  errorlevel 0,-305
  INCLUDE "d:\writing\pic_mast\code\p16cxx.inc"

;  Registers
Seg0    EQU     0x00C           ;  Value in the First Segment
Seg1    EQU     0x00D
Seg2    EQU     0x00E
Dig     EQU     0x00F           ;  Segment Currently Being Displayed
Count   EQU     0x010
Counthi EQU     0x011
_w      EQU     0x012           ;  Interrupt Status Save Values
_status EQU     0x013
Value   EQU     0x014           ;  Value To be Printed Out

 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF
                                ;  Note that the WatchDog Timer is OFF

  PAGE
;  Code for TEST6

  org    0

  goto   MainLine               ;  Skip Over Output Tables

  org    4                      ;  Interrupt Handler Address

  goto   Int                    ;  Skip Over the Output Tables

SEGMENTS_A                      ;  PORTA Outputs for the Segments
  addwf  PCL
  retlw  0x01C                  ;   0 - "0" - Ports RA2, RA3, RA4
  retlw  0x010                  ;   1 - "1" - Ports RA4
  retlw  0x014                  ;   2 - "2" - Ports RA2, RA4
  retlw  0x014                  ;   3 - "3" - Ports RA2, RA4
  retlw  0x018                  ;   4 - "4" - Ports RA3, RA4
  retlw  0x00C                  ;   5 - "5" - Ports RA2, RA3
  retlw  0x00C                  ;   6 - "6" - Ports RA2, RA3
  retlw  0x014                  ;   7 - "7" - Ports RA2, RA4
  retlw  0x01C                  ;   8 - "8" - All Port Bits
  retlw  0x01C                  ;   9 - "9" - Ports RA2, RA3, RA4
  retlw  0x00C                  ;  10 - "C" - Ports RA2, RA3
  retlw  0x01C                  ;  11 - "A" - Ports RA2, RA3, RA4
  retlw  0x008                  ;  12 - "L" - Ports RA3
  retlw  0                      ;  13 - "u" - Nothing from PORTA
  retlw  0                      ;  14 - "-" - Nothing from PORTA
  retlw  0x01C                  ;  15 - "^" - PORTS RA2, RA3, RA4

SEGMENTS_B                      ;  PORTB Outputs for the Segments
  addwf  PCL
  retlw  0x0E                   ;   0 - "0" - RB1, RB2, RB3
  retlw  8                      ;   1 - "1" - RB3
  retlw  7                      ;   2 - "2" - RB0, RB1, RB2
  retlw  0x0D                   ;   3 - "3" - RB0, RB2, RB3
  retlw  9                      ;   4 - "4" - RB0, RB3
  retlw  0x0D                   ;   5 - "5" - RB0, RB2, RB3
  retlw  0x0F                   ;   6 - "6" - RB0, RB1, RB2, RB3
  retlw  8                      ;   7 - "7" - RB3
  retlw  0x0F                   ;   8 - "8" - All Port Bits
  retlw  9                      ;   9 - "9" - RB0, RB3
  retlw  6                      ;  10 - "C" - RB1, RB2
  retlw  0x0B                   ;  11 - "A" - RB0, RB1, RB3
  retlw  6                      ;  12 - "L" - RB1, RB2
  retlw  0x0E                   ;  13 - "u" - RB1, RB2, RB3
  retlw  1                      ;  14 - "-" - RB0
  retlw  0                      ;  15 - "^" - Nothing from PORTB

Digits                          ;  Output Digits - Note that Top Bit Set
  addwf  PCL                    ;   High for the Resistor Value Read
  retlw  0x0C0                  ;  0 - First Digit
  retlw  0x0A0                  ;  1 - Second Digit
  retlw  0x090                  ;  2 - Third Digit

MainLine

  clrf   Value                  ;  Start with Value = 0 and Go Up
  clrf   Dig                    ;  Start at the First Digit

  clrf   Seg0                   ;  Reset the Segments because of Problems
  clrf   Seg1                   ;   found with EEPROM Writing
  clrf   Seg2

  clrf   PORTA                  ;  Make Sure the Ports can't Output Data
  clrf   PORTB

  bsf    STATUS, RP0            ;  Enable the Value Change LED
  movlw  0x03                   ;  Enable Output on PORTA.4:2
  movwf  TRISA & 0x07F
  movlw  0x080                  ;  Enable All Outputs except PORTB.7
  movwf  TRISB & 0x07F
  movlw  0x0D0                  ;  Use CLKOUT for the TMR and set at 2KHz
  movwf  OPTION_REG & 0x07F
  bcf    STATUS, RP0

  clrf   TMR0                   ;  Clear the Timer and Prescaler

  movlw  0x0A0                  ;  Enable the Timer Interrupt
  movwf  INTCON

Loop                            ;  Loop Around Here to Output the Sawtooth

  bcf    INTCON, GIE            ;  Turn Off Interrupts During Sensor Read
  bsf    STATUS, RP0            ;  Charge the Capacitor
  bcf    TRISB & 0x07F, 7
  movlw  0x032
  movwf  Value
  decfsz Value
   goto  $ - 1

  bsf    TRISB & 0x07F, 7       ;  Allow the Cap to Discharge
  bcf    STATUS, RP0

  clrf   Value                  ;  Now, Read the Value
TLoop                           ;  Wait for the Cap to Discharge through
  btfss  PORTB, 7               ;   Variable Resistor
   goto  HaveValue              ;  Jump Out if the Value == 0
  incfsz Value                  ;  Increment the Time
   goto  TLoop                  ;   Check again if No TimeOut

TooCold                         ;  Value is Too Cold

  movlw  13                     ;  TimeOut, Value is too Low (cold)
  movwf  Seg0                   ;   to Display
  movwf  Seg1
  movwf  Seg2

  goto   Display

HaveValue                       ;  We have Reached 0 - Can we Display it?

  movlw  25                     ;  Value Must be >= 25
  subwf  Value, w
  btfsc  STATUS, C              ;  "C" is the "Positive Flag"
   goto  GetReal                ;  Positive Flag is Set, Value is Okay

TooHot                          ;  Too Hot, Outside the Range

  movlw  15                     ;  Temperature is too high - Error
  movwf  Seg0
  movwf  Seg1
  movwf  Seg2

  goto   Display

GetReal                         ;  Now we have the Correct Value

  movwf  Value                  ;  Save the Offset
  call   GetTemp                ;  Get the Temperature from the Table
  movwf  Value                  ;  Save the Value Read In
  movlw  high $                 ;  Reset PCLATH to the Correct Value
  movwf  PCLATH

  call   SetSeg                 ;  Show what the Value is

Display                         ;  *Now* we can Display the Value

  bsf    INTCON, GIE            ;  Turn Interrupts Back On after Change

  call   Dlay                   ;  Dlay 1/2 Second to Show the Value

  goto   Loop


Int                             ;  Interrupt Handler

  movwf  _w                     ;  Save the Current Values
  swapf  STATUS, w
  movwf  _status

  movlw  0x020                  ;  Clear the Interrupt Handler
  movwf  INTCON

  movlw  Seg0                   ;  Output the Correct Segment
  addwf  Dig, w
  movwf  FSR                    ;  Use the FSR to Index into the Value

  movf   INDF, w                ;  Turn Off Digit O/P and Turn On Display
  call   SEGMENTS_B
  movwf  PORTB

  movf   INDF, w                ;  Now, Make Sure PORTA is Set Correctly
  call   SEGMENTS_A
  movwf  PORTA

  movf   Dig, w                 ;  Now, Everything Setup, Turn on the Digit
  call   Digits
  iorwf  PORTB

  incf   Dig                    ;  For the Next Interrupt, Point to the
  movlw  3                      ;   Next Digit in the Chain (or the First One)
  subwf  Dig, w
  btfsc  STATUS, C              ;  If Result is Negative, Don't Clear
   clrf  Dig

  swapf  _status, w             ;  Return from Interrupt with Valid Regs
  movwf  STATUS
  swapf  _w
  swapf  _w, w
  retfie


SetSeg                          ;  Load Seg0,1,2 from "Value"
                                ;  Do the Sign, Hundred Bit First
  btfss  Value, 7               ;  Do we have a Negative Number?
   goto  SS_Positive            ;  Nope, It's a Positive Number

  movlw  14                     ;  Display the "-" Sign
  movwf  Seg2                   ;  Put in the First Location

  movf   Value, w               ;  Convert to Positive Before Check
  sublw  0
  addlw  0 - 100                ;  Do we have a Displayable Negative Number
  btfss  STATUS, C              ;  If Result is Positive, then "yes"
   goto  SS_NegPos              ;  It's not, Make Postive and Show

  movlw  13                     ;  Negative > 99, Can't be Displayed
  movwf  Seg1                   ;  Reset the Segments
  movwf  Seg0

  goto   SS_End                 ;  Finished, Return

SS_NegPos                       ;  Number is > -100, Can Display

  addlw  100                    ;  Put it Back in it's Correct Range

  goto   SS_Display             ;  Now, Display it

SS_Positive                     ;  Positive Number, Set the High Value

  clrf   Seg2                   ;  Clear the MSDigit

  movlw  100                    ;  Are we Greater than 100?
  subwf  Value, w
  btfsc  STATUS, C              ;  Only Set the "1" if Value > 99
   goto  SS_100                 ;  Value is > 99, Treat it Specially

  movf   Value, w               ;  Put the Value in "w"

  goto   SS_Display             ;  Display It

SS_100                          ;  Value is Over 100, Set MSDigit and
                                ;   Set the Value in "w"
  incf   Seg2                   ;  Set the MSDigit

  movlw  100                    ;  Subtract 100 from Value
  subwf  Value, w               ;   And Leave the Result in "w" for Displaying

SS_Display                      ;  Display Contents of "w" in Seg1 & Seg2

  clrf   Seg1                   ;  Clear the 10's Digit

SS_Loop                         ;  Loop Here Until "w" < 10

  incf   Seg1                   ;  Increment the Segment

  addlw  0 - 10                 ;  Subtract 10 from the Current
  btfsc  STATUS, C              ;   Until we Have a Negative
   goto  SS_Loop

  decf   Seg1                   ;  Negative, Make it Positive
  addlw  10

  movwf  Seg0                   ;  Save the LSB

SS_End                          ;  All the Segments are Set, Return

  return


Dlay                            ;  One Third Second Delay

  movlw  0x081
  movwf  Count
  movlw  0x06D
  movwf  Counthi
Dlay_Loop
  decfsz Count
   goto  Dlay_Loop
  decfsz Counthi
   goto  Dlay_Loop

  return


  org    0x0300                 ;  Put the Temperature Table Here...

GetTemp

  movlw  high $                 ;  Make Sure PCLath is Set Properly
  movwf  PCLATH
  movf   Value, w               ;  Use the Value as an Offset
  addlw  -25
  addwf  PCL
                                ; Count
  retlw  50                     ;  25 - Count = 1 for each Degree
  retlw  49                     ;  26
  retlw  48                     ;  27
  retlw  47                     ;  28
  retlw  46                     ;  29
  retlw  45                     ;  30
  retlw  44                     ;  31
  retlw  43                     ;  32
  retlw  42                     ;  33
  retlw  41                     ;  34
  retlw  40                     ;  35
  retlw  39                     ;  36
  retlw  38                     ;  37
  retlw  37                     ;  38
  retlw  36                     ;  39
  retlw  35                     ;  40
  retlw  34                     ;  41
  retlw  33                     ;  42
  retlw  32                     ;  43 - Count = 2 for each Degree
  retlw  32                     ;  44
  retlw  31                     ;  45
  retlw  31                     ;  46
  retlw  31                     ;  47
  retlw  30                     ;  48
  retlw  30                     ;  49
  retlw  29                     ;  50
  retlw  29                     ;  51
  retlw  28                     ;  52
  retlw  28                     ;  53
  retlw  27                     ;  54
  retlw  27                     ;  55
  retlw  27                     ;  56
  retlw  26                     ;  57
  retlw  26                     ;  58
  retlw  25                     ;  59
  retlw  25                     ;  60
  retlw  24                     ;  61 - Count = 3 for each Degree
  retlw  24                     ;  62
  retlw  24                     ;  63
  retlw  23                     ;  64
  retlw  23                     ;  65
  retlw  23                     ;  66
  retlw  22                     ;  67
  retlw  22                     ;  68
  retlw  21                     ;  69
  retlw  21                     ;  70
  retlw  21                     ;  71
  retlw  20                     ;  72
  retlw  20                     ;  73
  retlw  20                     ;  74
  retlw  19                     ;  75
  retlw  19                     ;  76
  retlw  19                     ;  77
  retlw  18                     ;  78
  retlw  18                     ;  79
  retlw  18                     ;  80
  retlw  17                     ;  81
  retlw  17                     ;  82
  retlw  17                     ;  83
  retlw  16                     ;  84
  retlw  16                     ;  85
  retlw  16                     ;  86
  retlw  15                     ;  87
  retlw  15                     ;  88
  retlw  15                     ;  89
  retlw  14                     ;  90
  retlw  14                     ;  91
  retlw  14                     ;  92
  retlw  13                     ;  93 - Count = 4 for each Degree
  retlw  13                     ;  94
  retlw  13                     ;  95
  retlw  13                     ;  96
  retlw  12                     ;  97
  retlw  12                     ;  98
  retlw  12                     ;  99
  retlw  12                     ; 100
  retlw  11                     ; 101
  retlw  11                     ; 102
  retlw  11                     ; 103
  retlw  11                     ; 104
  retlw  10                     ; 105
  retlw  10                     ; 106
  retlw  10                     ; 107
  retlw  10                     ; 108
  retlw  9                      ; 109
  retlw  9                      ; 110
  retlw  9                      ; 111
  retlw  9                      ; 112
  retlw  8                      ; 113 - Count = 5 for each degree
  retlw  8                      ; 114
  retlw  8                      ; 115
  retlw  8                      ; 116
  retlw  8                      ; 117
  retlw  7                      ; 118
  retlw  7                      ; 119
  retlw  7                      ; 120
  retlw  7                      ; 121
  retlw  7                      ; 122
  retlw  6                      ; 123
  retlw  6                      ; 124
  retlw  6                      ; 125
  retlw  6                      ; 126
  retlw  6                      ; 127
  retlw  5                      ; 128 - Count = 6 for Each Degree
  retlw  5                      ; 129
  retlw  5                      ; 130
  retlw  5                      ; 131
  retlw  5                      ; 132
  retlw  5                      ; 133
  retlw  4                      ; 134
  retlw  4                      ; 135
  retlw  4                      ; 136
  retlw  4                      ; 137
  retlw  4                      ; 138
  retlw  4                      ; 139
  retlw  3                      ; 140
  retlw  3                      ; 141
  retlw  3                      ; 142
  retlw  3                      ; 143
  retlw  3                      ; 144
  retlw  3                      ; 145
  retlw  2                      ; 146
  retlw  2                      ; 147
  retlw  2                      ; 148
  retlw  2                      ; 149
  retlw  2                      ; 150
  retlw  2                      ; 151
  retlw  1                      ; 152
  retlw  1                      ; 153
  retlw  1                      ; 154
  retlw  1                      ; 155
  retlw  1                      ; 156
  retlw  1                      ; 157
  retlw  0                      ; 158 - Count = 7 for each degree
  retlw  0                      ; 159
  retlw  0                      ; 160
  retlw  0                      ; 161
  retlw  0                      ; 162
  retlw  0                      ; 163
  retlw  0                      ; 164
  retlw  -1                     ; 165
  retlw  -1                     ; 166
  retlw  -1                     ; 167
  retlw  -1                     ; 168
  retlw  -1                     ; 169
  retlw  -1                     ; 170
  retlw  -1                     ; 171
  retlw  -2                     ; 172
  retlw  -2                     ; 173
  retlw  -2                     ; 174
  retlw  -2                     ; 175
  retlw  -2                     ; 176
  retlw  -2                     ; 177
  retlw  -2                     ; 178
  retlw  -3                     ; 179 - Count = 8 for each degree
  retlw  -3                     ; 180
  retlw  -3                     ; 181
  retlw  -3                     ; 182
  retlw  -3                     ; 183
  retlw  -3                     ; 184
  retlw  -3                     ; 185
  retlw  -3                     ; 186
  retlw  -4                     ; 187
  retlw  -4                     ; 188
  retlw  -4                     ; 189
  retlw  -4                     ; 190
  retlw  -4                     ; 191
  retlw  -4                     ; 192
  retlw  -4                     ; 193
  retlw  -4                     ; 194
  retlw  -5                     ; 195 - Count = 9 for each degree
  retlw  -5                     ; 196
  retlw  -5                     ; 197
  retlw  -5                     ; 198
  retlw  -5                     ; 199
  retlw  -5                     ; 200
  retlw  -5                     ; 201
  retlw  -5                     ; 202
  retlw  -5                     ; 203
  retlw  -6                     ; 204
  retlw  -6                     ; 205
  retlw  -6                     ; 206
  retlw  -6                     ; 207
  retlw  -6                     ; 208
  retlw  -6                     ; 209
  retlw  -6                     ; 210
  retlw  -6                     ; 211
  retlw  -6                     ; 212
  retlw  -6                     ; 213
  retlw  -7                     ; 214
  retlw  -7                     ; 215
  retlw  -7                     ; 216
  retlw  -7                     ; 217
  retlw  -7                     ; 218
  retlw  -7                     ; 219
  retlw  -7                     ; 220
  retlw  -7                     ; 221
  retlw  -7                     ; 222
  retlw  -8                     ; 223
  retlw  -8                     ; 224
  retlw  -8                     ; 225
  retlw  -8                     ; 226
  retlw  -8                     ; 227
  retlw  -8                     ; 228
  retlw  -8                     ; 229
  retlw  -8                     ; 230
  retlw  -8                     ; 231
  retlw  -8                     ; 232
  retlw  -8                     ; 233
  retlw  -9                     ; 234
  retlw  -9                     ; 235
  retlw  -9                     ; 236
  retlw  -9                     ; 237
  retlw  -9                     ; 238
  retlw  -9                     ; 239
  retlw  -9                     ; 240
  retlw  -9                     ; 241
  retlw  -9                     ; 242
  retlw  -9                     ; 243
  retlw  -9                     ; 244
  retlw  -9                     ; 245
  retlw  -10                    ; 246
  retlw  -10                    ; 247
  retlw  -10                    ; 248
  retlw  -10                    ; 249
  retlw  -10                    ; 250
  retlw  -10                    ; 251
  retlw  -10                    ; 252
  retlw  -10                    ; 253
  retlw  -10                    ; 254
  retlw  -10                    ; 255



  end
