 title  "TEST7 - Read the EEPROM and Show with Increment."
;
;  This Program Reads the initial EEPROM Address and displays them on
;   the LED Displays each 1/3 Second.  If one of the buttons is pressed,
;   the Count is incremented (both in the file Registers and in EEPROM).
;
;            --6--
;           |     |
;           5     1
;           |     |
;            --7--
;           |     |
;           4     2
;           |     |
;            --3--
;
;  Each LED Segment is connected to a PIC pin via a 220 Ohm Resistor
;   Except for the Segment connected to RA0, in which the 220 Ohm Resistor
;   is attached to Vdd and the PIC Pin pulls it low to turn OFF the LED.
;
;  The operation of each different 7 Segment Display is controlled by a
;   2N70000 N-Channel MOSFET attached to the common cathod to the display.
;   The gate to the controlling MOSFET is connected directly to an output
;   PIN on the PIC.
;
;  Hardware Notes:
;   Reset is tied directly to Vcc and PWRT is Enabled.
;   RA2 is Connected to Segment 6
;   RA3 is Connected to Segment 5
;   RA4 is Connected to Segment 1
;   RB0 is Connected to Segment 7
;   RB1 is Connected to Segment 4
;   RB2 is Connected to Segment 3
;   RB3 is Connected to Segment 2
;   RB4 is Connected to the Left Most Digit
;   RB5 is Connected to the Middle Digit
;   RB6 is Connected to the Right Most Digit
;   RA0 is Connected to the "Up" Pin
;   RA1 is Connected to the "Dwn" Pin
;
;  The Various Numbers use the Segments Below:
;   "0" - Segments 1, 2, 3, 4, 5, 6  Ports RA4, RB3, RB2, RB1, RA3, RA2
;   "1" - Segments 1, 2              Ports RA4, RB3
;   "2" - Segments 6, 1, 7, 4, 3     Ports RA2, RA4, RB0, RB1, RB2
;   "3" - Segments 6, 1, 2, 3, 7     Ports RA2, RA4, RB3, RB2, RB0
;   "4" - Segmemts 7, 1, 2, 5        Ports RB0, RA4, RB3, RA3
;   "5" - Segments 6, 5, 7, 2, 3     Ports RA2, RA3, RB0, RB3, RB2
;   "6" - Segments 6, 5, 4, 3, 2, 7  Ports RA2, RA3, RB1, RB2, RB3, RB0
;   "7" - Segments 6, 1, 2           Ports RA2, RA4, RB3
;   "8" - ALL Segments               ALL Port Bits
;   "9" - Segments 6, 1, 2, 7, 5     Ports RA2, RA4, RB3, RB0, RA3
;   "C" - Segments 6, 5, 4, 3        Ports RA2, RA3, RB1, RB2
;   "A" - Segments 6, 1, 2, 4, 5, 7  Ports RA2, RA4, RB3, RB1, RA3, RB0
;   "L" - Segments 5, 4, 3           Ports RA3, RB1, RB2
;   "u" - Segments 2, 3, 4           Ports RB3, RB2, RB1
;   "-" - Segments 7                 Ports RB0
;   "^" - Segments 1, 5, 6           Ports RA4, RA3, RA2
;
;  Myke Predko
;  96.07.21
;
  LIST P=16C84, R=DEC
  errorlevel 0,-305
  INCLUDE "d:\writing\pic_mast\code\p16cxx.inc"

;  Registers
Seg0    EQU     0x00C           ;  Value in the First Segment
Seg1    EQU     0x00D
Seg2    EQU     0x00E
Dig     EQU     0x00F           ;  Segment Currently Being Displayed
Count   EQU     0x010
Counthi EQU     0x011
_w      EQU     0x012           ;  Interrupt Status Save Values
_status EQU     0x013
Value   EQU     0x014           ;  Value To be Printed Out

;  EEPROM Registers
CK1     EQU     0               ;  Check Values for Reading
CK2     EQU     1
SVD     EQU     2

 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF
                                ;  Note that the WatchDog Timer is OFF

  PAGE
;  Code for TEST7

  org    0

  goto   MainLine               ;  Skip Over Output Tables

  org    4                      ;  Interrupt Handler Address

  goto   Int                    ;  Skip Over the Output Tables

SEGMENTS_A                      ;  PORTA Outputs for the Segments
  addwf  PCL
  retlw  0x01C                  ;   0 - "0" - Ports RA2, RA3, RA4
  retlw  0x010                  ;   1 - "1" - Ports RA4
  retlw  0x014                  ;   2 - "2" - Ports RA2, RA4
  retlw  0x014                  ;   3 - "3" - Ports RA2, RA4
  retlw  0x018                  ;   4 - "4" - Ports RA3, RA4
  retlw  0x00C                  ;   5 - "5" - Ports RA2, RA3
  retlw  0x00C                  ;   6 - "6" - Ports RA2, RA3
  retlw  0x014                  ;   7 - "7" - Ports RA2, RA4
  retlw  0x01C                  ;   8 - "8" - All Port Bits
  retlw  0x01C                  ;   9 - "9" - Ports RA2, RA3, RA4
  retlw  0x00C                  ;  10 - "C" - Ports RA2, RA3
  retlw  0x01C                  ;  11 - "A" - Ports RA2, RA3, RA4
  retlw  0x008                  ;  12 - "L" - Ports RA3
  retlw  0                      ;  13 - "u" - Nothing from PORTA
  retlw  0                      ;  14 - "-" - Nothing from PORTA
  retlw  0x01C                  ;  15 - "^" - PORTS RA2, RA3, RA4

SEGMENTS_B                      ;  PORTB Outputs for the Segments
  addwf  PCL
  retlw  0x0E                   ;   0 - "0" - RB1, RB2, RB3
  retlw  8                      ;   1 - "1" - RB3
  retlw  7                      ;   2 - "2" - RB0, RB1, RB2
  retlw  0x0D                   ;   3 - "3" - RB0, RB2, RB3
  retlw  9                      ;   4 - "4" - RB0, RB3
  retlw  0x0D                   ;   5 - "5" - RB0, RB2, RB3
  retlw  0x0F                   ;   6 - "6" - RB0, RB1, RB2, RB3
  retlw  8                      ;   7 - "7" - RB3
  retlw  0x0F                   ;   8 - "8" - All Port Bits
  retlw  9                      ;   9 - "9" - RB0, RB3
  retlw  6                      ;  10 - "C" - RB1, RB2
  retlw  0x0B                   ;  11 - "A" - RB0, RB1, RB3
  retlw  6                      ;  12 - "L" - RB1, RB2
  retlw  0x0E                   ;  13 - "u" - RB1, RB2, RB3
  retlw  1                      ;  14 - "-" - RB0
  retlw  0                      ;  15 - "^" - Nothing from PORTB

Digits                          ;  Output Digits - Note that Top Bit Set
  addwf  PCL                    ;   High for the Resistor Value Read
  retlw  0x0C0                  ;  0 - First Digit
  retlw  0x0A0                  ;  1 - Second Digit
  retlw  0x090                  ;  2 - Third Digit

MainLine

  clrf   Value                  ;  Start with Value = 0 and Go Up
  clrf   Dig                    ;  Start at the First Digit

  clrf   Seg0                   ;  Clear the Segment Values for Initial
  clrf   Seg1                   ;   Table Read
  clrf   Seg2

  clrf   PORTA                  ;  Make Sure the Ports can't Output Data
  clrf   PORTB

  bsf    STATUS, RP0            ;  Enable the Value Change LED
  movlw  0x03                   ;  Enable Output on PORTA.4:2
  movwf  TRISA & 0x07F
  movlw  0x080                  ;  Enable All Outputs except PORTB.7
  movwf  TRISB & 0x07F
  movlw  0x0D0                  ;  Use CLKOUT for the TMR and set at 2KHz
  movwf  OPTION_REG & 0x07F
  bcf    STATUS, RP0

  clrf   TMR0                   ;  Clear the Timer and Prescaler

  movlw  0x0A0                  ;  Enable the Timer Interrupt
  movwf  INTCON

  movlw  CK1                    ;  Is the EEPROM Set?
  call   EEREAD
  addlw  0 - 0x0AA              ;  Do we Have AA/55 at the Start?
  btfss  STATUS, Z
   goto  SetFirstValue          ;  Nope, Set up the EEPROM
  movlw  CK2
  call   EEREAD                 ;  Check the Second Byte
  addlw  0 - 0x055
  btfsc  STATUS, Z
   goto  LoopStart              ;  Everything is Fine, Show the Value

SetFirstValue

  movlw  0x0AA
  movwf  EEDATA
  movlw  CK1                    ;  Now, Write in the Expected Values
  movwf  EEADR
  call   EEWRITE

  movlw  0x055
  movwf  EEDATA
  movlw  CK2
  movwf  EEADR
  call   EEWRITE

  movlw  0
  movwf  EEDATA
  movlw  SVD
  movwf  EEADR
  call   EEWRITE

LoopStart                       ;  Read the Initial Value and Put in "Value"
  movlw  SVD                    ;  Get the Saved Value
  call   EEREAD
  movwf  Value

Loop                            ;  Loop Around Here to Output the Sawtooth

  bcf    INTCON, GIE
  call   SetSeg                 ;  Show what the Value is
  bsf    INTCON, GIE            ;  Turn Interrupts Back On after Change

  ifndef Debug
  call   Dlay                   ;  Dlay 1/2 Second to Show the Value
  endif

  btfsc  PORTA, 0               ;  Are we going up?
   goto  ChkDwn                 ;  No, See if we're going down

  incf   Value                  ;  Yes, Increment the Value

  goto   SaveValue              ;  And Save it

ChkDwn                          ;  Check the Value if going down
  btfsc  PORTA, 1
   goto  Loop

  decf   Value                  ;  Decrement the Value (Go Down)

SaveValue                       ;  Save the New Value for Next Time

  movf   Value, w
  movwf  EEDATA
  movlw  SVD                    ;  Save the Value in the EEPROM
  movwf  EEADR
  call   EEWRITE

  goto   Loop


Int                             ;  Interrupt Handler

  movwf  _w                     ;  Save the Current Values
  swapf  STATUS, w
  movwf  _status

  movlw  0x020                  ;  Clear the Interrupt Handler
  movwf  INTCON

  movlw  Seg0                   ;  Output the Correct Segment
  addwf  Dig, w
  movwf  FSR                    ;  Use the FSR to Index into the Value

  movf   INDF, w                ;  Turn Off Digit O/P and Turn On Display
  call   SEGMENTS_B
  movwf  PORTB

  movf   INDF, w                ;  Now, Make Sure PORTA is Set Correctly
  call   SEGMENTS_A
  movwf  PORTA

  movf   Dig, w                 ;  Now, Everything Setup, Turn on the Digit
  call   Digits
  iorwf  PORTB

  incf   Dig                    ;  For the Next Interrupt, Point to the
  movlw  3                      ;   Next Digit in the Chain (or the First One)
  subwf  Dig, w
  btfsc  STATUS, C              ;  If Result is Negative, Don't Clear
   clrf  Dig

  swapf  _status, w             ;  Return from Interrupt with Valid Regs
  movwf  STATUS
  swapf  _w
  swapf  _w, w
  retfie


SetSeg                          ;  Load Seg0,1,2 from "Value"
                                ;  Do the Sign, Hundred Bit First
  btfss  Value, 7               ;  Do we have a Negative Number?
   goto  SS_Positive            ;  Nope, It's a Positive Number

  movlw  14                     ;  Display the "-" Sign
  movwf  Seg2                   ;  Put in the First Location

  movf   Value, w               ;  Convert to Positive Before Check
  sublw  0
  addlw  0 - 100                ;  Do we have a Displayable Negative Number
  btfss  STATUS, C              ;  If Result is Positive, then "yes"
   goto  SS_NegPos              ;  It's not, Make Postive and Show

  movlw  13                     ;  Negative > 99, Can't be Displayed
  movwf  Seg1                   ;  Reset the Segments
  movwf  Seg0

  goto   SS_End                 ;  Finished, Return

SS_NegPos                       ;  Number is > -100, Can Display

  addlw  100                    ;  Put it Back in it's Correct Range

  goto   SS_Display             ;  Now, Display it

SS_Positive                     ;  Positive Number, Set the High Value

  clrf   Seg2                   ;  Clear the MSDigit

  movlw  100                    ;  Are we Greater than 100?
  subwf  Value, w
  btfsc  STATUS, C              ;  Only Set the "1" if Value > 99
   goto  SS_100                 ;  Value is > 99, Treat it Specially

  movf   Value, w               ;  Put the Value in "w"

  goto   SS_Display             ;  Display It

SS_100                          ;  Value is Over 100, Set MSDigit and
                                ;   Set the Value in "w"
  incf   Seg2                   ;  Set the MSDigit

  movlw  100                    ;  Subtract 100 from Value
  subwf  Value, w               ;   And Leave the Result in "w" for Displaying

SS_Display                      ;  Display Contents of "w" in Seg1 & Seg2

  clrf   Seg1                   ;  Clear the 10's Digit

SS_Loop                         ;  Loop Here Until "w" < 10

  incf   Seg1                   ;  Increment the Segment

  addlw  0 - 10                 ;  Subtract 10 from the Current
  btfsc  STATUS, C              ;   Until we Have a Negative
   goto  SS_Loop

  decf   Seg1                   ;  Negative, Make it Positive
  addlw  10

  movwf  Seg0                   ;  Save the LSB

SS_End                          ;  All the Segments are Set, Return

  return


Dlay                            ;  One Third Second Delay

  movlw  0x081
  movwf  Count
  movlw  0x06D
  movwf  Counthi
Dlay_Loop
  decfsz Count
   goto  Dlay_Loop
  decfsz Counthi
   goto  Dlay_Loop

  return


EEREAD                          ;  Read the EEPROM and Return Data in "w"
  movwf  EEADR                  ;  Set the EEADR from "w"
  bsf    STATUS, RP0            ;  Work Takes Place in Page 1
  bsf    EECON1 & 0x07F, RD
  bcf    STATUS, RP0
  movf   EEDATA & 0x07F , w     ;  Read Value Ends up in "w"
  return


EEWRITE
  bcf    INTCON, GIE
  bsf    STATUS, RP0
  clrf   EECON1 & 0x07F
  bsf    EECON1 & 0x07F, WREN
  movlw  0x055                  ;  Do the Standard Write Control
  movwf  EECON2 & 0x07F
  movlw  0x0AA
  movwf  EECON2 & 0x07F
  bsf    EECON1 & 0x07F, WR
  bcf    EECON1 & 0x07F, WREN
EELoop                          ;  Loop Here Until Write Carried Out
  clrwdt
  btfsc  EECON1 & 0x07F, WR
   goto  EELoop

  bcf    STATUS, RP0
  bsf    INTCON, GIE

  return                        ;  Return to the Caller


  end
